/*
 * captouch - A demostration program for using capacitive touch sensors
 * Copyright (C) 2012 Andreas AUer <andreas.auer@emicrotec.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <avr/io.h>
#include <util/delay.h>

#include <string.h>
#include <stdlib.h>

#include "captouch.h"
#include "usart.h"

//------------------------------------------------------------------------------
void touchFilterLP(TouchKey *key)
{
    key->filter_counter++;
    key->filter_counter &= (FILTER_LENGTH - 1);

    key->filter_value -= key->filter_buffer[key->filter_counter];
    key->filter_value += key->current_value;
    key->filter_buffer[key->filter_counter] = key->current_value;
    
    key->base_value = key->filter_value >> FILTER_SHIFT;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
void touchInit(TouchConfig *config)
{
     MCUCR |= (1u << PUD);
     
     *(config->base + 1) |=  (config->mask_snsk | config->mask_sns);
     *(config->base + 2) &= ~(config->mask_snsk | config->mask_sns);
     
     config->first_time = 1;
}

//------------------------------------------------------------------------------
void touchSense(TouchConfig *config, TouchKey *key)
{
    unsigned short i = MAX_SENSE_CYCLE; 
    unsigned char  measure = 0;
    unsigned char  mask_sns = config->mask_sns;
    unsigned char  a;
    unsigned char  b;
    
#ifndef LP_FILTER
    key->base_value = key->current_value;
#endif
//    ATOMIC_BLOCK(ATOMIC_FORCEON)
    {
        TOUCH_DDR  &= ~(config->mask_snsk | config->mask_sns);
        TOUCH_PORT &= ~(config->mask_snsk | config->mask_sns);
        
        a = TOUCH_DDR;
        b = TOUCH_PORT;
        
        do
        {
            TOUCH_DDR  = a;
            TOUCH_PORT = b | mask_sns;
            TOUCH_DDR  = a | mask_sns;
            TOUCH_DDR  = a;
            TOUCH_PORT = b;
            TOUCH_DDR  = a | config->mask_snsk;
            i--;
            if(i == 0)
            {
                break;
            }
            measure = TOUCH_PIN & config->mask_sns;
            if(measure)
            {
                key->current_value = i;
                break;
            }
        }
        while(1);

        *(config->base + 1) |=  (config->mask_snsk | config->mask_sns);
        *(config->base + 2) &= ~(config->mask_snsk | config->mask_sns);
    }
    
    if(config->first_time)
    {
        config->first_time--;
        for(i=0; i<FILTER_LENGTH; i++)
        {
            key->filter_buffer[i] = key->current_value;
            key->filter_value = FILTER_LENGTH * key->current_value;
            touchFilterLP(key);
        }
    }
    
#ifdef LP_FILTER
    if((config->filter_counter++) == config->filter_time)
    {
        config->filter_counter = 0;
        touchFilterLP(key);
    }
#endif
    if(key->current_value < key->base_value)
    {
        key->base_value = key->current_value;
        key->current_state = TOUCH_UP;
    }
    else if(abs(key->current_value - key->base_value) > key->delta)
    {
        usartPutInt(key->current_value, 10);
        usartPutStr("\t");
        usartPutInt(key->base_value, 10);
        usartPutStr("\t");
        usartPutIntLine(abs(key->current_value - key->base_value), 10);
        key->current_state = TOUCH_DOWN;
    }
    else
    {
        key->current_state = TOUCH_UP;
    }
}

