/*
 * captouch - A demostration program for using capacitive touch sensors
 * Copyright (C) 2012 Andreas AUer <andreas.auer@emicrotec.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include <string.h>

#define __delay_cycles(n)     __builtin_avr_delay_cycles(n)
#define __enable_interrupt()  sei()

#include "usart.h"
#include "captouch.h"

#define TICKS_PER_MS                1000u
#define MEASUREMENT_PERIOD_MS       25u

/*----------------------------------------------------------------------------
                                static variables
----------------------------------------------------------------------------*/

/* flag set by timer ISR when it's time to measure touch */
static volatile uint8_t time_to_measure_touch = 0u;

/* current time, set by timer ISR */
static volatile uint16_t current_time_ms_touch = 0u;

//------------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect)
{
    /*  set flag: it's time to measure touch    */
    time_to_measure_touch = 1u;

    /*  update the current time */
    current_time_ms_touch += MEASUREMENT_PERIOD_MS;
}

//------------------------------------------------------------------------------
static void init_system(void)
{
    DDRB  |= _BV(PB0) | _BV(PB1) | _BV(PB2);
    PORTB |= 0x07;
}

//------------------------------------------------------------------------------
static void init_timer_isr(void)
{
    /*  set timer compare value (how often timer ISR will fire) */
    OCR1A = (TICKS_PER_MS * MEASUREMENT_PERIOD_MS);

    /*  enable timer ISR */
    TIMSK |= (1u << OCIE1A);

    /*  timer prescaler = system clock / 8  */
    TCCR1B |= (1u << CS11);

    /*  timer mode = CTC (count up to compare value, then reset)    */
    TCCR1B |= (1u << WGM12);
}

//------------------------------------------------------------------------------
#if 0
static int sense(void)
{
    unsigned short i = 1024; 
    unsigned char  measure = 0;
    unsigned char  a, b;
    
//    ATOMIC_BLOCK(ATOMIC_FORCEON)
    {
        DDRC  = 0x00;
        PORTC = 0x00;
        
        a = DDRC;
        b = PORTC;
        
        do
        {
            DDRC   = a;
            PORTC |= 0x02;
            DDRC  |= 0x02;
            _delay_us(2);
            DDRC   = a;
            PORTC  = b;
            DDRC   = a | 0x01;
            _delay_us(2);
            i--;
            if(i == 0)
            {
                break;
            }
            measure = PINC & 0x02;
            if(measure)
            {
                break;
            }
        }
        while(1);

        DDRC |= _BV(PC1);
        DDRC |= _BV(PC0);
        PORTC = 0x00;
    }    
    return i;
}
#endif
//------------------------------------------------------------------------------
int main(void)
{
    unsigned char  i;
    TouchKey keys[3];
    TouchConfig config[3] = {
    {
        .base      = &PINC,
        .mask_snsk = 0x1,
        .mask_sns  = 0x2,
        .filter_time = 80,
        .filter_counter = 0,
    },
    {
        .base      = &PINC,
        .mask_snsk = 0x4,
        .mask_sns  = 0x8,
        .filter_time = 40,
        .filter_counter = 0,
    },
    {
        .base      = &PINC,
        .mask_snsk = 0x10,
        .mask_sns  = 0x20,
        .filter_time = 40,
        .filter_counter = 0,
    },
    };

    memset(keys, 0, sizeof(keys));
    for(i=0; i<3; i++)
    {
        keys[i].delta = 20;
    }
    init_system();
    touchInit(&(config[0]));
    touchInit(&(config[1]));
    touchInit(&(config[2]));

    usartInit(25);
    usartPutLine("QTouch experiment!");
    
    init_timer_isr();
    
    sei();
    
    while(1)
    {
        if(time_to_measure_touch)
        {
            time_to_measure_touch = 0u;
            //usartPutIntLine(sense(), 10);
            //continue;

            touchSense(&(config[0]), &(keys[0]));
            //touchSense(&(config[1]), &(keys[1]));
            //touchSense(&(config[2]), &(keys[2]));
           
            for(i=0; i<3; i++)
            {
                if(keys[i].current_state == TOUCH_DOWN)
                {
                    PORTB &= ~_BV(i);
                }
                else
                {
                    PORTB |= _BV(i);
                }
            }
/*           
            if(keys[1].delta < -10)
            {
                PORTB &= ~_BV(PB0);
                usartPutStr("  "); 
                usartPutInt(keys[1].delta,10);
                usartPutStr("\t"); 
                usartPutInt(keys[1].current_value,10);
                usartPutStr("\t"); 
                usartPutIntLine(keys[1].base_value,10);
            }
            else
            {
                PORTB |= _BV(PB0);
            }
*/
        }
    }
    
    
    
    return 0;
}
