/*
 * captouch - A demostration program for using capacitive touch sensors
 * Copyright (C) 2012 Andreas AUer <andreas.auer@emicrotec.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <avr/io.h>

#include <stdlib.h>

#include "usart.h"

//----------------------------------------------------------------------
void usartInit(unsigned int ubrr)
{
	UBRRH = (unsigned char)(ubrr >> 8);
	UBRRL = (unsigned char) ubrr;
	
	UCSRB = _BV(RXEN) | _BV(TXEN);
	UCSRC = _BV(URSEL) | _BV(UCSZ1) | _BV(UCSZ0);
}

//----------------------------------------------------------------------
void usartPutByte(unsigned char data)
{
  loop_until_bit_is_set(UCSRA,UDRE);
  UDR = data;
}

//----------------------------------------------------------------------
void usartPutChar(unsigned char data)
{
	
  if(data == '\n')
  {
    loop_until_bit_is_set(UCSRA,UDRE);
    UDR = '\r';
  }
  loop_until_bit_is_set(UCSRA,UDRE);
  UDR = data;
}

//----------------------------------------------------------------------
void usartPutStr(char *str)
{
	unsigned int i = 0;
	
	for(i=0;str[i];i++)
		usartPutChar((unsigned char) str[i]);
}

//----------------------------------------------------------------------
unsigned char usartGetChar(void)
{
  unsigned char data = 0;

//  while(!(UCSRA & _BV(RXC)));

  if(UCSRA & _BV(RXC))
  {
    data = UDR;
  }

	return data;
}
//----------------------------------------------------------------------
void usartPutLine(char *str)
{
    usartPutStr(str);
    usartPutChar('\n');
}

//----------------------------------------------------------------------
void usartPutInt(int value, int base)
{
    char buf[16];
    itoa(value, buf, base);
    usartPutStr(buf);
}

//----------------------------------------------------------------------
void usartPutIntLine(int value, int base)
{
    char buf[16];
    itoa(value, buf, base);
    usartPutLine(buf);
}

