/*
 * captouch - A demostration program for using capacitive touch sensors
 * Copyright (C) 2012 Andreas AUer <andreas.auer@emicrotec.at>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/** 
 * @file usart.h
 * @author Andreas Auer
 * @brief USART Kommunikation
 * 
 * Dieser Softwareteil dient zur Kommunikation ber die 
 * serielle Schnittstelle des Atmel Controllers. Einerseits
 * dient sie zu DEBUG Zwecken, vorallem aber findet die 
 * Software in der Kommunikation mit dem 2. Mikrocontroller
 * Verwendung!
 */

#ifndef __USART_H
#define __USART_H

/**
 * Der Funktion muss als Parameter ein Wert fr das 
 * UBRR Register bergeben werden. Durch diesen Wert
 * wird die entsprechende Baudrate eingestellt bzw. 
 * generiert!
 * @param unsigned int ubrr Parameter fr UBRR Register
 */
void usartInit(unsigned int ubrr);

/**
 * Funktion schreibt EIN Zeichen bzw. Byte auf die 
 * serielle Schnittstelle. Zuvor muss USART natrlich
 * initialisiert worden sein!
 * @param unsigned char data Byte, das auf USART geschrieben
 *               werden soll!
 */
void usartPutChar(unsigned char data);

/**
 * Gibt eine ganze Zeichenkette auf der seriellen Schnittstelle
 * aus. Schnittstelle muss ebenfalls initialisiert sein.
 * Diese Funktion ist nur fr Strings (ASCII Zeichenketten, die
 * mit '\0' abgeschlossen sind) sinnvoll, da NULL als 
 * Abbruchsbedingung fr die Funktion verwendet wird.
 * Die Funktion verwendet usartPutChar als Hilfsfunktion!
 * @param char *str Zeichenkette, die ausgegeben werden soll!
 */
void usartPutStr(char *str);

void usartPutLine(char *str);
void usartPutInt(int value, int base);
void usartPutIntLine(int value, int base);


/**
 * Liest ein Zeichen von der seriellen Schnittstelle. USART muss
 * initialisiert sein!
 * @return Liefert 0, wenn keine Daten empfangen wurden, ansonsten 
 *         ungleich 0.
 */
unsigned char usartGetChar(void);

void usartPutByte(unsigned char data);

#endif
